/**
 * Created by jiangcheng on 2016/08/28.
 */
(function ($, window, factory) {
    // 判断是否支持模块定义
    var hasDefine = (typeof define === 'function');
    if (hasDefine) {
        //获取对象
        var exports = factory(fox, fox.callproxy);
        //定义模块
        define(exports);
        //安装插件(兼容非模块的访问方式)
        window.fox.device = exports;
    } else {
        //获取对象
        var exports = factory(fox, fox.callproxy);
        //安装插件
        window.fox.device = exports;
    }

}($, window, function (fox, callproxy) {

    //外设访问器
    var device = {

        //计数
        _count:0,

        /**
         * 显示cover
         * @private
         */
        _showCover:function _showCover() {
            //计算重复调用次数
            this._count += 1;
            //只有count为1，才允许绘制界面
            if (this._count > 1) {
                return;
            }
            //打开cover
            this.coverId=fox.layer.open({
                type: fox.layer.Cover
                ,content: '外设读取中...',
                shadeClose:false
            });
        },

        /**
         * 隐藏cover
         * @private
         */
        _hideCover:function _hideCover() {
            //计算重复调用次数
            this._count -= 1;
            //只有count为0，才允许关闭界面
            if (this._count > 0) {
                return;
            }
            //隐藏
            fox.layer.close(this.coverId);
        },

        /**
         * 调用外设
         * @param type
         * @param action
         * @param params
         * @param callback
         */
        call: function (type, action, params, callback) {

            //设置默认值
            if (params == undefined) {
                params = {};
            }
            var reqData;
            //请求数据转换为字符串
            if (fox.type(params) == "object") {
                reqData = JSON.stringify(params);
            } else if(fox.type(params)=="array"){
                reqData={data:params};
                reqData = JSON.stringify(reqData);
            } else {
                reqData = params;
            }

            try {
                //生成回调方法
                var callbackName = "";
                if (callback) {
                    var _this = this;
                    callbackName = callproxy.register(callback, function () {
                        //关闭cover
                        _this._hideCover();
                        return false;
                    });
                    // this._showCover();
                }

                //调用外设
                _FOX_PROXY_DEVICE.call(type, action, reqData, callbackName);
            }catch(ex){
                //打印日志
                fox.logger.error(ex.message,ex);
                if(callback) {
                    //关闭cover
                    this._hideCover();
                    //回调函数
                    callback(2, ex.message, "");
                }

            }
        },

        /**
         * 获取外设配置
         * @param callback
         */
        getDeviceConfig: function (callback) {
            //生成回调方法
            var callbackName = "";
            if (callback) {
                callbackName = callproxy.register(callback);
            }
            //获取外设配置
            _FOX_PROXY_DEVICE.getDeviceConfig(callbackName);
        },

        /**
         * 获取外设映射
         * @param callback
         */
        getDeviceMap: function (callback) {
            //生成回调方法
            var callbackName = "";
            if (callback) {
                callbackName = callproxy.register(callback);
            }
            //获取外设配置
            _FOX_PROXY_DEVICE.getDeviceMap(callbackName);
        },

        /**
         * 设置外设映射
         * @param data
         * @param callback
         */
        setDeviceMap: function (data, callback) {
            //设置默认值
            if (data == undefined) {
                data = {};
            }
            var reqData;
            //请求数据转换为字符串
            if (fox.type(data) == "object") {
                reqData = JSON.stringify(data);
            } else {
                reqData = data;
            }

            //生成回调方法
            var callbackName = "";
            if (callback) {
                callbackName = callproxy.register(callback);
            }
            //获取外设配置
            _FOX_PROXY_DEVICE.setDeviceMap(reqData, callbackName);
        },

        /**
         * 调用摄像头
         * @param cameraType
         * @param callback
         * @param params
         */
        callCamera: function (cameraType,callback, params) {
            this.call("camera", cameraType, params, callback);
        },

        /**
         * 调用签名
         * @param action
         * @param callback
         * @param params
         */
        callSign: function (callback, params) {
            this.call("sign", "sign", params, callback);
        },

        /**
         * 调用密码键盘
         * @param action
         * @param callback
         * @param params
         */
        callPin: function (action, callback, params) {
            this.call("pin", action, params, callback);
        },
        /**
         * 调用文件压缩解压
         * @param action
         * @param callback
         * @param params
         */
        callZip: function(action,params,callback){
            this.call("zip",action,params,callback);
        },
        /**
         * 调用手势解锁
         * @param action
         * @param callback
         * @param params
         */
        callGesture: function(action,params,callback){
            this.call("ysgesturelock",action,params,callback);
        },
        /**
         * 数据库操作
         * @param action
         * @param callback
         * @param params
         */
        callDatabase: function(action,params,callback){
            this.call("database",action,params,callback);
        },
        //阿里支付
        callAliPay: function(action,params,callback){
            device.call("callAliPay",action,params,callback);
        },

        //微信支付
        callWXPay: function(action,params,callback){
            device.call("callWXPay",action,params,callback);
        },
        /**
         * 一网通支付
         * @param cameraType
         * @param callback
         * @param params
         */
        callCMBPay: function (action,params,callback){
            this.call("callCMBPay", action, params, callback);
        },

        //腾讯云登录
        callTRTC: function(action,params,callback){
            device.call("callTRTC",action,params,callback);
        },

        //获取定位
        callLocation: function(action,params,callback){
            device.call("callLocation",action,params,callback);
        },
        callZZRS: function(action,params,callback){
            device.call("callZZRS",action,params,callback);
        },
        callCmbO: function(action,params,callback){
            device.call("callCmbO",action,params,callback);
        },


        /**
         * 跳转appStore
         * @param cameraType
         * @param callback
         * @param params
         */
        callCheckUpdate: function (action,callback, params) {
            this.call("checkUpdate", action, params, callback);
        },


        onekeyshare: {
            mobshare: function (action,params, callback) {
                device.call("mobshare", action, params, callback);
            }
        }

    };
    //返回对象
    return device;

}));
